@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 // Package main implements a CLI tool for generating
 // long-term credentials.
 package main
@@ -9,7 +12,7 @@ import (
 	"os"
 	"time"
 
-	"github.com/pion/turn/v2"
+	"github.com/pion/turn/v4"
 )
 
 // Outputs username & password according to the
@@ -22,6 +25,7 @@ func main() {
 	if showHelp != nil && *showHelp {
 		log.Println("Usage:")
 		log.Println("$ lt-cred-generator | xargs go run examples/turn-client/udp/main.go -host localhost -ping=true -user=")
+
 		return
 	}
 
@@ -30,10 +34,11 @@ func main() {
 	}
 
 	u, p, _ := turn.GenerateLongTermCredentials(*authSecret, time.Minute)
-	if _, err := os.Stdout.WriteString(fmt.Sprintf("%s=%s", u, p)); err != nil { // for use with xargs
+
+	if _, err := fmt.Fprintf(os.Stdout, "%s=%s", u, p); err != nil { // For use with xargs
 		log.Panicf("Failed to write to stdout: %s", err)
 	}
-	if _, err := os.Stderr.WriteString("\n"); err != nil { // ignored by xargs
+	if _, err := fmt.Fprintf(os.Stderr, "\n"); err != nil { // Ignored by xargs
 		log.Panicf("Failed to write to stderr: %s", err)
 	}
 }
