@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 // Package main implements a TURN client using UDP
 package main
 
@@ -10,10 +13,10 @@ import (
 	"time"
 
 	"github.com/pion/logging"
-	"github.com/pion/turn/v2"
+	"github.com/pion/turn/v4"
 )
 
-func main() {
+func main() { //nolint:cyclop
 	host := flag.String("host", "", "TURN Server name.")
 	port := flag.Int("port", 3478, "Listening port.")
 	user := flag.String("user", "", "A pair of username and password (e.g. \"user=pass\")")
@@ -32,7 +35,7 @@ func main() {
 	cred := strings.SplitN(*user, "=", 2)
 
 	// TURN client won't create a local listening socket by itself.
-	conn, err := net.ListenPacket("udp4", "0.0.0.0:0")
+	conn, err := net.ListenPacket("udp4", "0.0.0.0:0") // nolint: noctx
 	if err != nil {
 		log.Panicf("Failed to listen: %s", err)
 	}
@@ -93,7 +96,7 @@ func main() {
 	}
 }
 
-func doPingTest(client *turn.Client, relayConn net.PacketConn) error {
+func doPingTest(client *turn.Client, relayConn net.PacketConn) error { //nolint:cyclop
 	// Send BindingRequest to learn our external IP
 	mappedAddr, err := client.SendBindingRequest()
 	if err != nil {
@@ -101,7 +104,7 @@ func doPingTest(client *turn.Client, relayConn net.PacketConn) error {
 	}
 
 	// Set up pinger socket (pingerConn)
-	pingerConn, err := net.ListenPacket("udp4", "0.0.0.0:0")
+	pingerConn, err := net.ListenPacket("udp4", "0.0.0.0:0") // nolint: noctx
 	if err != nil {
 		log.Panicf("Failed to listen: %s", err)
 	}
