@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 // Package main implements a TURN server with logging.
 package main
 
@@ -12,12 +15,12 @@ import (
 	"strconv"
 	"syscall"
 
-	"github.com/pion/stun"
-	"github.com/pion/turn/v2"
+	"github.com/pion/stun/v3"
+	"github.com/pion/turn/v4"
 )
 
 // stunLogger wraps a PacketConn and prints incoming/outgoing STUN packets
-// This pattern could be used to capture/inspect/modify data as well
+// This pattern could be used to capture/inspect/modify data as well.
 type stunLogger struct {
 	net.PacketConn
 }
@@ -64,7 +67,7 @@ func main() {
 	// Create a UDP listener to pass into pion/turn
 	// pion/turn itself doesn't allocate any UDP sockets, but lets the user pass them in
 	// this allows us to add logging, storage or modify inbound/outbound traffic
-	udpListener, err := net.ListenPacket("udp4", "0.0.0.0:"+strconv.Itoa(*port))
+	udpListener, err := net.ListenPacket("udp4", "0.0.0.0:"+strconv.Itoa(*port)) // nolint: noctx
 	if err != nil {
 		log.Panicf("Failed to create TURN server listener: %s", err)
 	}
@@ -76,15 +79,16 @@ func main() {
 		usersMap[kv[1]] = turn.GenerateAuthKey(kv[1], *realm, kv[2])
 	}
 
-	s, err := turn.NewServer(turn.ServerConfig{
+	server, err := turn.NewServer(turn.ServerConfig{
 		Realm: *realm,
 		// Set AuthHandler callback
 		// This is called every time a user tries to authenticate with the TURN server
 		// Return the key for that user, or false when no user is found
-		AuthHandler: func(username string, realm string, srcAddr net.Addr) ([]byte, bool) {
+		AuthHandler: func(username string, realm string, srcAddr net.Addr) ([]byte, bool) { // nolint: revive
 			if key, ok := usersMap[username]; ok {
 				return key, true
 			}
+
 			return nil, false
 		},
 		// PacketConnConfigs is a list of UDP Listeners and the configuration around them
@@ -92,8 +96,10 @@ func main() {
 			{
 				PacketConn: &stunLogger{udpListener},
 				RelayAddressGenerator: &turn.RelayAddressGeneratorStatic{
-					RelayAddress: net.ParseIP(*publicIP), // Claim that we are listening on IP passed by user (This should be your Public IP)
-					Address:      "0.0.0.0",              // But actually be listening on every interface
+					// Claim that we are listening on IP passed by user (This should be your Public IP)
+					RelayAddress: net.ParseIP(*publicIP),
+					// But actually be listening on every interface
+					Address: "0.0.0.0",
 				},
 			},
 		},
@@ -107,7 +113,7 @@ func main() {
 	signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM)
 	<-sigs
 
-	if err = s.Close(); err != nil {
+	if err = server.Close(); err != nil {
 		log.Panic(err)
 	}
 }
