@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 // Package main implements a TURN server with TLS support
 package main
 
@@ -12,7 +15,7 @@ import (
 	"strconv"
 	"syscall"
 
-	"github.com/pion/turn/v2"
+	"github.com/pion/turn/v4"
 )
 
 func main() {
@@ -33,6 +36,7 @@ func main() {
 	cer, err := tls.LoadX509KeyPair(*certFile, *keyFile)
 	if err != nil {
 		log.Println(err)
+
 		return
 	}
 
@@ -45,6 +49,7 @@ func main() {
 	})
 	if err != nil {
 		log.Println(err)
+
 		return
 	}
 
@@ -55,15 +60,16 @@ func main() {
 		usersMap[kv[1]] = turn.GenerateAuthKey(kv[1], *realm, kv[2])
 	}
 
-	s, err := turn.NewServer(turn.ServerConfig{
+	server, err := turn.NewServer(turn.ServerConfig{
 		Realm: *realm,
 		// Set AuthHandler callback
 		// This is called every time a user tries to authenticate with the TURN server
 		// Return the key for that user, or false when no user is found
-		AuthHandler: func(username string, realm string, srcAddr net.Addr) ([]byte, bool) {
+		AuthHandler: func(username string, realm string, srcAddr net.Addr) ([]byte, bool) { // nolint: revive
 			if key, ok := usersMap[username]; ok {
 				return key, true
 			}
+
 			return nil, false
 		},
 		// ListenerConfig is a list of Listeners and the configuration around them
@@ -86,7 +92,7 @@ func main() {
 	signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM)
 	<-sigs
 
-	if err = s.Close(); err != nil {
+	if err = server.Close(); err != nil {
 		log.Panic(err)
 	}
 }
