@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 //go:build !js
 // +build !js
 
@@ -10,9 +13,9 @@ import (
 	"testing"
 	"time"
 
-	"github.com/pion/stun"
-	"github.com/pion/turn/v2/internal/ipnet"
-	"github.com/pion/turn/v2/internal/proto"
+	"github.com/pion/stun/v3"
+	"github.com/pion/turn/v4/internal/ipnet"
+	"github.com/pion/turn/v4/internal/proto"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -43,237 +46,230 @@ func TestAllocation(t *testing.T) {
 }
 
 func subTestGetPermission(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	addr2, err := net.ResolveUDPAddr("udp", "127.0.0.1:3479")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	addr3, err := net.ResolveUDPAddr("udp", "127.0.0.2:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
-	p := &Permission{
+	perms := &Permission{
 		Addr: addr,
 	}
-	p2 := &Permission{
+	perms2 := &Permission{
 		Addr: addr2,
 	}
-	p3 := &Permission{
+	perms3 := &Permission{
 		Addr: addr3,
 	}
 
-	a.AddPermission(p)
-	a.AddPermission(p2)
-	a.AddPermission(p3)
+	alloc.AddPermission(perms)
+	alloc.AddPermission(perms2)
+	alloc.AddPermission(perms3)
 
-	foundP1 := a.GetPermission(addr)
-	assert.Equal(t, p, foundP1, "Should keep the first one.")
+	foundP1 := alloc.GetPermission(addr)
+	assert.Equal(t, perms, foundP1, "Should keep the first one.")
 
-	foundP2 := a.GetPermission(addr2)
-	assert.Equal(t, p, foundP2, "Second one should be ignored.")
+	foundP2 := alloc.GetPermission(addr2)
+	assert.Equal(t, perms, foundP2, "Second one should be ignored.")
 
-	foundP3 := a.GetPermission(addr3)
-	assert.Equal(t, p3, foundP3, "Permission with another IP should be found")
+	foundP3 := alloc.GetPermission(addr3)
+	assert.Equal(t, perms3, foundP3, "Permission with another IP should be found")
 }
 
 func subTestAddPermission(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	p := &Permission{
 		Addr: addr,
 	}
 
-	a.AddPermission(p)
-	assert.Equal(t, a, p.allocation, "Permission's allocation should be the adder.")
+	alloc.AddPermission(p)
+	assert.Equal(t, alloc, p.allocation, "Permission's allocation should be the adder.")
 
-	foundPermission := a.GetPermission(p.Addr)
+	foundPermission := alloc.GetPermission(p.Addr)
 	assert.Equal(t, p, foundPermission)
 }
 
 func subTestRemovePermission(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	p := &Permission{
 		Addr: addr,
 	}
 
-	a.AddPermission(p)
+	alloc.AddPermission(p)
 
-	foundPermission := a.GetPermission(p.Addr)
+	foundPermission := alloc.GetPermission(p.Addr)
 	assert.Equal(t, p, foundPermission, "Got permission is not same as the the added.")
 
-	a.RemovePermission(p.Addr)
+	alloc.RemovePermission(p.Addr)
 
-	foundPermission = a.GetPermission(p.Addr)
+	foundPermission = alloc.GetPermission(p.Addr)
 	assert.Nil(t, foundPermission, "Got permission should be nil after removed.")
 }
 
 func subTestAddChannelBind(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	c := NewChannelBind(proto.MinChannelNumber, addr, nil)
 
-	err = a.AddChannelBind(c, proto.DefaultLifetime)
+	err = alloc.AddChannelBind(c, proto.DefaultLifetime)
 	assert.Nil(t, err, "should succeed")
-	assert.Equal(t, a, c.allocation, "allocation should be the caller.")
+	assert.Equal(t, alloc, c.allocation, "allocation should be the caller.")
 
 	c2 := NewChannelBind(proto.MinChannelNumber+1, addr, nil)
-	err = a.AddChannelBind(c2, proto.DefaultLifetime)
+	err = alloc.AddChannelBind(c2, proto.DefaultLifetime)
 	assert.NotNil(t, err, "should failed with conflicted peer address")
 
 	addr2, _ := net.ResolveUDPAddr("udp", "127.0.0.1:3479")
 	c3 := NewChannelBind(proto.MinChannelNumber, addr2, nil)
-	err = a.AddChannelBind(c3, proto.DefaultLifetime)
+	err = alloc.AddChannelBind(c3, proto.DefaultLifetime)
 	assert.NotNil(t, err, "should fail with conflicted number.")
 }
 
 func subTestGetChannelByNumber(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	c := NewChannelBind(proto.MinChannelNumber, addr, nil)
 
-	_ = a.AddChannelBind(c, proto.DefaultLifetime)
+	_ = alloc.AddChannelBind(c, proto.DefaultLifetime)
 
-	existChannel := a.GetChannelByNumber(c.Number)
+	existChannel := alloc.GetChannelByNumber(c.Number)
 	assert.Equal(t, c, existChannel)
 
-	notExistChannel := a.GetChannelByNumber(proto.MinChannelNumber + 1)
+	notExistChannel := alloc.GetChannelByNumber(proto.MinChannelNumber + 1)
 	assert.Nil(t, notExistChannel, "should be nil for not existed channel.")
 }
 
 func subTestGetChannelByAddr(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	c := NewChannelBind(proto.MinChannelNumber, addr, nil)
 
-	_ = a.AddChannelBind(c, proto.DefaultLifetime)
+	_ = alloc.AddChannelBind(c, proto.DefaultLifetime)
 
-	existChannel := a.GetChannelByAddr(c.Peer)
+	existChannel := alloc.GetChannelByAddr(c.Peer)
 	assert.Equal(t, c, existChannel)
 
 	addr2, _ := net.ResolveUDPAddr("udp", "127.0.0.1:3479")
-	notExistChannel := a.GetChannelByAddr(addr2)
+	notExistChannel := alloc.GetChannelByAddr(addr2)
 	assert.Nil(t, notExistChannel, "should be nil for not existed channel.")
 }
 
 func subTestRemoveChannelBind(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	addr, err := net.ResolveUDPAddr("udp", "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	c := NewChannelBind(proto.MinChannelNumber, addr, nil)
 
-	_ = a.AddChannelBind(c, proto.DefaultLifetime)
+	_ = alloc.AddChannelBind(c, proto.DefaultLifetime)
 
-	a.RemoveChannelBind(c.Number)
+	alloc.RemoveChannelBind(c.Number)
 
-	channelByNumber := a.GetChannelByNumber(c.Number)
+	channelByNumber := alloc.GetChannelByNumber(c.Number)
 	assert.Nil(t, channelByNumber)
 
-	channelByAddr := a.GetChannelByAddr(c.Peer)
+	channelByAddr := alloc.GetChannelByAddr(c.Peer)
 	assert.Nil(t, channelByAddr)
 }
 
 func subTestAllocationRefresh(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 
 	var wg sync.WaitGroup
 	wg.Add(1)
-	a.lifetimeTimer = time.AfterFunc(proto.DefaultLifetime, func() {
+	alloc.lifetimeTimer = time.AfterFunc(proto.DefaultLifetime, func() {
 		wg.Done()
 	})
-	a.Refresh(0)
+	alloc.Refresh(0)
 	wg.Wait()
 
-	// lifetimeTimer has expired
-	assert.False(t, a.lifetimeTimer.Stop())
+	// LifetimeTimer has expired
+	assert.False(t, alloc.lifetimeTimer.Stop())
 }
 
 func subTestAllocationClose(t *testing.T) {
+	t.Helper()
+
 	network := "udp"
 
-	l, err := net.ListenPacket(network, "0.0.0.0:0")
-	if err != nil {
-		panic(err)
-	}
+	l, err := net.ListenPacket(network, "0.0.0.0:0") // nolint: noctx
+	assert.NoError(t, err)
 
-	a := NewAllocation(nil, nil, nil)
-	a.RelaySocket = l
-	// add mock lifetimeTimer
-	a.lifetimeTimer = time.AfterFunc(proto.DefaultLifetime, func() {})
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
+	alloc.RelaySocket = l
+	// Add mock lifetimeTimer
+	alloc.lifetimeTimer = time.AfterFunc(proto.DefaultLifetime, func() {})
 
-	// add channel
+	// Add channel
 	addr, err := net.ResolveUDPAddr(network, "127.0.0.1:3478")
-	if err != nil {
-		t.Fatalf("failed to resolve: %s", err)
-	}
+	assert.NoError(t, err)
 
 	c := NewChannelBind(proto.MinChannelNumber, addr, nil)
-	_ = a.AddChannelBind(c, proto.DefaultLifetime)
+	_ = alloc.AddChannelBind(c, proto.DefaultLifetime)
 
-	// add permission
-	a.AddPermission(NewPermission(addr, nil))
+	// Add permission
+	alloc.AddPermission(NewPermission(addr, nil))
 
-	err = a.Close()
-	assert.Nil(t, err, "should succeed")
-	assert.True(t, isClose(a.RelaySocket), "should be closed")
+	assert.Nil(t, alloc.Close(), "should succeed")
+	assert.True(t, isClose(alloc.RelaySocket), "should be closed")
 }
 
 func subTestPacketHandler(t *testing.T) {
+	t.Helper()
+
 	network := "udp"
 
-	m, _ := newTestManager()
+	manager, _ := newTestManager()
 
-	// turn server initialization
-	turnSocket, err := net.ListenPacket(network, "127.0.0.1:0")
-	if err != nil {
-		panic(err)
-	}
+	// TURN server initialization
+	turnSocket, err := net.ListenPacket(network, "127.0.0.1:0") // nolint: noctx
+	assert.NoError(t, err)
 
-	// client listener initialization
-	clientListener, err := net.ListenPacket(network, "127.0.0.1:0")
-	if err != nil {
-		panic(err)
-	}
+	// Client listener initialization
+	clientListener, err := net.ListenPacket(network, "127.0.0.1:0") // nolint: noctx
+	assert.NoError(t, err)
 
 	dataCh := make(chan []byte)
-	// client listener read data
+	// Client listener read data
 	go func() {
 		buffer := make([]byte, rtpMTU)
 		for {
@@ -286,84 +282,79 @@ func subTestPacketHandler(t *testing.T) {
 		}
 	}()
 
-	a, err := m.CreateAllocation(&FiveTuple{
+	alloc, err := manager.CreateAllocation(&FiveTuple{
 		SrcAddr: clientListener.LocalAddr(),
 		DstAddr: turnSocket.LocalAddr(),
-	}, turnSocket, 0, proto.DefaultLifetime)
+	}, turnSocket, 0, proto.DefaultLifetime, "", "")
 
-	assert.Nil(t, err, "should succeed")
+	assert.NoError(t, err, "should succeed")
 
-	peerListener1, err := net.ListenPacket(network, "127.0.0.1:0")
-	if err != nil {
-		panic(err)
-	}
+	peerListener1, err := net.ListenPacket(network, "127.0.0.1:0") // nolint: noctx
+	assert.NoError(t, err)
 
-	peerListener2, err := net.ListenPacket(network, "127.0.0.1:0")
-	if err != nil {
-		panic(err)
-	}
+	peerListener2, err := net.ListenPacket(network, "127.0.0.1:0") // nolint: noctx
+	assert.NoError(t, err)
 
-	// add permission with peer1 address
-	a.AddPermission(NewPermission(peerListener1.LocalAddr(), m.log))
-	// add channel with min channel number and peer2 address
-	channelBind := NewChannelBind(proto.MinChannelNumber, peerListener2.LocalAddr(), m.log)
-	_ = a.AddChannelBind(channelBind, proto.DefaultLifetime)
+	// Add permission with peer1 address
+	alloc.AddPermission(NewPermission(peerListener1.LocalAddr(), manager.log))
+	// Add channel with min channel number and peer2 address
+	channelBind := NewChannelBind(proto.MinChannelNumber, peerListener2.LocalAddr(), manager.log)
+	_ = alloc.AddChannelBind(channelBind, proto.DefaultLifetime)
 
-	_, port, _ := ipnet.AddrIPPort(a.RelaySocket.LocalAddr())
+	_, port, _ := ipnet.AddrIPPort(alloc.RelaySocket.LocalAddr())
 	relayAddrWithHostStr := fmt.Sprintf("127.0.0.1:%d", port)
 	relayAddrWithHost, _ := net.ResolveUDPAddr(network, relayAddrWithHostStr)
 
-	// test for permission and data message
+	// Test for permission and data message
 	targetText := "permission"
 	_, _ = peerListener1.WriteTo([]byte(targetText), relayAddrWithHost)
 	data := <-dataCh
 
-	// resolve stun data message
+	// Resolve stun data message
 	assert.True(t, stun.IsMessage(data), "should be stun message")
 
 	var msg stun.Message
-	err = stun.Decode(data, &msg)
-	assert.Nil(t, err, "decode data to stun message failed")
+	assert.NoError(t, stun.Decode(data, &msg), "decode data to stun message failed")
 
 	var msgData proto.Data
-	err = msgData.GetFrom(&msg)
-	assert.Nil(t, err, "get data from stun message failed")
+	assert.NoError(t, msgData.GetFrom(&msg), "get data from stun message failed")
 	assert.Equal(t, targetText, string(msgData), "get message doesn't equal the target text")
 
-	// test for channel bind and channel data
+	// Test for channel bind and channel data
 	targetText2 := "channel bind"
 	_, _ = peerListener2.WriteTo([]byte(targetText2), relayAddrWithHost)
 	data = <-dataCh
 
-	// resolve channel data
+	// Resolve channel data
 	assert.True(t, proto.IsChannelData(data), "should be channel data")
 
 	channelData := proto.ChannelData{
 		Raw: data,
 	}
-	err = channelData.Decode()
-	assert.Nil(t, err, fmt.Sprintf("channel data decode with error: %v", err))
+	assert.NoError(t, channelData.Decode(), fmt.Sprintf("channel data decode with error: %v", err))
 	assert.Equal(t, channelBind.Number, channelData.Number, "get channel data's number is invalid")
 	assert.Equal(t, targetText2, string(channelData.Data), "get data doesn't equal the target text.")
 
-	// listeners close
-	_ = m.Close()
+	// Listeners close
+	_ = manager.Close()
 	_ = clientListener.Close()
 	_ = peerListener1.Close()
 	_ = peerListener2.Close()
 }
 
 func subTestResponseCache(t *testing.T) {
-	a := NewAllocation(nil, nil, nil)
+	t.Helper()
+
+	alloc := NewAllocation(nil, nil, EventHandler{}, nil)
 	transactionID := [stun.TransactionIDSize]byte{1, 2, 3}
 	responseAttrs := []stun.Setter{
 		&proto.Lifetime{
 			Duration: proto.DefaultLifetime,
 		},
 	}
-	a.SetResponseCache(transactionID, responseAttrs)
+	alloc.SetResponseCache(transactionID, responseAttrs)
 
-	cacheID, cacheAttr := a.GetResponseCache()
+	cacheID, cacheAttr := alloc.GetResponseCache()
 	assert.Equal(t, transactionID, cacheID)
 	assert.Equal(t, responseAttrs, cacheAttr)
 }
