@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 //go:build !js
 // +build !js
 
@@ -5,14 +8,13 @@ package server
 
 import (
 	"net"
-	"sync"
 	"testing"
 	"time"
 
 	"github.com/pion/logging"
-	"github.com/pion/stun"
-	"github.com/pion/turn/v2/internal/allocation"
-	"github.com/pion/turn/v2/internal/proto"
+	"github.com/pion/stun/v3"
+	"github.com/pion/turn/v4/internal/allocation"
+	"github.com/pion/turn/v4/internal/proto"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -24,17 +26,13 @@ func TestAllocationLifeTime(t *testing.T) {
 
 		m := &stun.Message{}
 		lifetimeDuration := allocationLifeTime(m)
-
-		if lifetimeDuration != proto.DefaultLifetime {
-			t.Errorf("Allocation lifetime should be default time duration")
-		}
-
+		assert.Equal(t, proto.DefaultLifetime, lifetimeDuration,
+			"Allocation lifetime should be default time duration")
 		assert.NoError(t, lifetime.AddTo(m))
 
 		lifetimeDuration = allocationLifeTime(m)
-		if lifetimeDuration != lifetime.Duration {
-			t.Errorf("Expect lifetimeDuration is %s, but %s", lifetime.Duration, lifetimeDuration)
-		}
+		assert.Equal(t, lifetime.Duration, lifetimeDuration,
+			"Allocation lifetime should be equal to the one set in the message")
 	})
 
 	// If lifetime is bigger than maximumLifetime
@@ -47,55 +45,56 @@ func TestAllocationLifeTime(t *testing.T) {
 		_ = lifetime.AddTo(m2)
 
 		lifetimeDuration := allocationLifeTime(m2)
-		if lifetimeDuration != proto.DefaultLifetime {
-			t.Errorf("Expect lifetimeDuration is %s, but %s", proto.DefaultLifetime, lifetimeDuration)
-		}
+		assert.Equal(t, proto.DefaultLifetime, lifetimeDuration)
 	})
 
 	t.Run("DeletionZeroLifetime", func(t *testing.T) {
-		l, err := net.ListenPacket("udp4", "0.0.0.0:0")
+		conn, err := net.ListenPacket("udp4", "0.0.0.0:0") // nolint: noctx
 		assert.NoError(t, err)
 		defer func() {
-			assert.NoError(t, l.Close())
+			assert.NoError(t, conn.Close())
 		}()
 
 		logger := logging.NewDefaultLoggerFactory().NewLogger("turn")
 
 		allocationManager, err := allocation.NewManager(allocation.ManagerConfig{
-			AllocatePacketConn: func(network string, requestedPort int) (net.PacketConn, net.Addr, error) {
-				conn, listenErr := net.ListenPacket(network, "0.0.0.0:0")
+			AllocatePacketConn: func(network string, _ int) (net.PacketConn, net.Addr, error) {
+				con, listenErr := net.ListenPacket(network, "0.0.0.0:0") // nolint: noctx
 				if err != nil {
 					return nil, nil, listenErr
 				}
 
-				return conn, conn.LocalAddr(), nil
+				return con, con.LocalAddr(), nil
 			},
-			AllocateConn: func(network string, requestedPort int) (net.Conn, net.Addr, error) {
+			AllocateConn: func(string, int) (net.Conn, net.Addr, error) {
 				return nil, nil, nil
 			},
 			LeveledLogger: logger,
 		})
 		assert.NoError(t, err)
 
-		staticKey := []byte("ABC")
-		r := Request{
-			AllocationManager: allocationManager,
-			Nonces:            &sync.Map{},
-			Conn:              l,
-			SrcAddr:           &net.UDPAddr{IP: net.ParseIP("127.0.0.1"), Port: 5000},
-			Log:               logger,
-			AuthHandler: func(username string, realm string, srcAddr net.Addr) (key []byte, ok bool) {
-				return staticKey, true
-			},
-		}
-		r.Nonces.Store(string(staticKey), time.Now())
-
-		fiveTuple := &allocation.FiveTuple{SrcAddr: r.SrcAddr, DstAddr: r.Conn.LocalAddr(), Protocol: allocation.UDP}
-
-		_, err = r.AllocationManager.CreateAllocation(fiveTuple, r.Conn, 0, time.Hour)
+		nonceHash, err := NewShortNonceHash(0)
+		assert.NoError(t, err)
+		staticKey, err := nonceHash.Generate()
 		assert.NoError(t, err)
 
-		assert.NotNil(t, r.AllocationManager.GetAllocation(fiveTuple))
+		req := Request{
+			AllocationManager: allocationManager,
+			NonceHash:         nonceHash,
+			Conn:              conn,
+			SrcAddr:           &net.UDPAddr{IP: net.ParseIP("127.0.0.1"), Port: 5000},
+			Log:               logger,
+			AuthHandler: func(string, string, net.Addr) (key []byte, ok bool) {
+				return []byte(staticKey), true
+			},
+		}
+
+		fiveTuple := &allocation.FiveTuple{SrcAddr: req.SrcAddr, DstAddr: req.Conn.LocalAddr(), Protocol: allocation.UDP}
+
+		_, err = req.AllocationManager.CreateAllocation(fiveTuple, req.Conn, 0, time.Hour, "", "")
+		assert.NoError(t, err)
+
+		assert.NotNil(t, req.AllocationManager.GetAllocation(fiveTuple))
 
 		m := &stun.Message{}
 		assert.NoError(t, (proto.Lifetime{}).AddTo(m))
@@ -104,7 +103,7 @@ func TestAllocationLifeTime(t *testing.T) {
 		assert.NoError(t, (stun.Realm(staticKey)).AddTo(m))
 		assert.NoError(t, (stun.Username(staticKey)).AddTo(m))
 
-		assert.NoError(t, handleRefreshRequest(r, m))
-		assert.Nil(t, r.AllocationManager.GetAllocation(fiveTuple))
+		assert.NoError(t, handleRefreshRequest(req, m))
+		assert.Nil(t, req.AllocationManager.GetAllocation(fiveTuple))
 	})
 }
